@@ -1,11 +1,8 @@
-/**************************************************************************
- *   Copyright (C) 2019 by Simone Gaiarin <simgunz@gmail.com>              *
- *                                                                         *
- *   This program is free software; you can redistribute it and/or modify  *
- *   it under the terms of the GNU General Public License as published by  *
- *   the Free Software Foundation; either version 2 of the License, or     *
- *   (at your option) any later version.                                   *
- **************************************************************************/
+/*
+    SPDX-FileCopyrightText: 2019 Simone Gaiarin <simgunz@gmail.com>
+
+    SPDX-License-Identifier: GPL-2.0-or-later
+*/
 
 #include "annotationactionhandler.h"
 
@@ -14,6 +11,7 @@
 #include <QColorDialog>
 #include <QFileInfo>
 #include <QFontDialog>
+#include <QMenu>
 #include <QPainter>
 #include <QPen>
 
@@ -24,12 +22,14 @@
 #include <KParts/MainWindow>
 #include <KSelectAction>
 #include <KToolBar>
+#include <kwidgetsaddons_version.h>
 
 // local includes
 #include "annotationwidgets.h"
 #include "guiutils.h"
 #include "pageview.h"
 #include "pageviewannotator.h"
+#include "settings.h"
 #include "toggleactionmenu.h"
 
 class AnnotationActionHandlerPrivate
@@ -59,6 +59,7 @@ public:
         , aAdvancedSettings(nullptr)
         , aHideToolBar(nullptr)
         , aShowToolBar(nullptr)
+        , aToolBarVisibility(nullptr)
         , aCustomStamp(nullptr)
         , aCustomWidth(nullptr)
         , aCustomOpacity(nullptr)
@@ -66,19 +67,20 @@ public:
         , currentInnerColor(QColor())
         , currentFont(QFont())
         , currentWidth(-1)
-        , selectedTool(-1)
+        , selectedBuiltinTool(-1)
         , textToolsEnabled(false)
     {
     }
 
     QAction *selectActionItem(KSelectAction *aList, QAction *aCustomCurrent, double value, const QList<double> &defaultValues, const QIcon &icon, const QString &label);
+
     /**
      * @short Adds a custom stamp annotation action to the stamp list when the stamp is not a default stamp
      *
-     * When stampIconName cannot be found among the default stamps, this method creates a new action
-     * for the custom stamp annotation and adds it to the stamp action combo box. If a custom action
-     * is already present in the list, it is removed before adding the new custom action. If stampIconName
-     * matches a default stamp, any existing stamp annotation action is removed.
+     * When @p stampIconName cannot be found among the default stamps, this method creates a new action
+     * for the custom stamp annotation and adds it to the stamp action combo box.
+     * If a custom action is already present in the list, it is removed before adding the new custom action.
+     * If @p stampIconName matches a default stamp, any existing custom stamp annotation action is removed.
      */
     void maybeUpdateCustomStampAction(const QString &stampIconName);
     void parseTool(int toolId);
@@ -96,6 +98,9 @@ public:
     void slotSetColor(AnnotationColor colorType, const QColor &color = QColor());
     void slotSelectAnnotationFont();
     void slotToolBarVisibilityChanged(bool checked);
+    bool isQuickToolAction(QAction *aTool);
+    bool isQuickToolStamp(int toolId);
+    void ephemeralStampWarning();
 
     AnnotationActionHandler *q;
 
@@ -106,7 +111,7 @@ public:
     QActionGroup *agTools;
     QAction *agLastAction;
 
-    KSelectAction *aQuickTools;
+    ToggleActionMenu *aQuickTools;
     ToggleActionMenu *aGeomShapes;
     ToggleActionMenu *aStamp;
     QAction *aAddToQuickTools;
@@ -119,7 +124,8 @@ public:
     QAction *aFont;
     QAction *aAdvancedSettings;
     QAction *aHideToolBar;
-    KToggleAction *aShowToolBar;
+    QAction *aShowToolBar;
+    KToggleAction *aToolBarVisibility;
 
     QAction *aCustomStamp;
     QAction *aCustomWidth;
@@ -130,7 +136,7 @@ public:
     QFont currentFont;
     int currentWidth;
 
-    int selectedTool;
+    int selectedBuiltinTool;
     bool textToolsEnabled;
 };
 
@@ -339,13 +345,27 @@ void AnnotationActionHandlerPrivate::populateQuickAnnotations()
     }
 
     const QList<int> numberKeys = {Qt::Key_1, Qt::Key_2, Qt::Key_3, Qt::Key_4, Qt::Key_5, Qt::Key_6, Qt::Key_7, Qt::Key_8, Qt::Key_9, Qt::Key_0};
+    const bool isFirstTimePopulated = aQuickTools->menu()->actions().count() == 0;
 
+    // to be safe and avoid undefined states of the currently selected quick annotation
+    if (isQuickToolAction(agTools->checkedAction())) {
+        q->deselectAllAnnotationActions();
+    }
+
+    const QList<QAction *> quickToolActions = aQuickTools->menu()->actions();
+    for (QAction *action : quickToolActions) {
+        if (action->isCheckable()) {
+            aQuickTools->removeAction(action);
+            delete action;
+        }
+    }
+    QAction *aSeparator = aQuickTools->menu()->actions().first();
     textQuickTools.clear();
-    aQuickTools->removeAllActions();
 
     int favToolId = 1;
     QList<int>::const_iterator shortcutNumber = numberKeys.begin();
     QDomElement favToolElement = annotator->quickTool(favToolId);
+    QList<QAction *> quickTools;
     while (!favToolElement.isNull()) {
         QString itemText = favToolElement.attribute(QStringLiteral("name"));
         if (favToolElement.attribute(QStringLiteral("default"), QStringLiteral("false")) == QLatin1String("true")) {
@@ -355,28 +375,41 @@ void AnnotationActionHandlerPrivate::populateQuickAnnotations()
             itemText = PageViewAnnotator::defaultToolName(favToolElement);
         }
         QIcon toolIcon = QIcon(PageViewAnnotator::makeToolPixmap(favToolElement));
-        QAction *annFav = new QAction(toolIcon, itemText, q);
-        aQuickTools->addAction(annFav);
+        QAction *annFav = new KToggleAction(toolIcon, itemText, q);
+        aQuickTools->insertAction(aSeparator, annFav);
+        agTools->addAction(annFav);
+        quickTools.append(annFav);
         if (shortcutNumber != numberKeys.end())
             annFav->setShortcut(QKeySequence(*(shortcutNumber++)));
-        QObject::connect(annFav, &QAction::triggered, q, [this, favToolId]() { slotQuickToolSelected(favToolId); });
-
+        QObject::connect(annFav, &KToggleAction::toggled, q, [this, favToolId](bool checked) {
+            if (checked) {
+                slotQuickToolSelected(favToolId);
+            }
+        });
         QDomElement engineElement = favToolElement.firstChildElement(QStringLiteral("engine"));
         if (engineElement.attribute(QStringLiteral("type")) == QStringLiteral("TextSelector")) {
             textQuickTools.append(annFav);
             annFav->setEnabled(textToolsEnabled);
         }
-
         favToolElement = annotator->quickTool(++favToolId);
     }
-    QAction *separator = new QAction();
-    separator->setSeparator(true);
-    aQuickTools->addAction(separator);
-    // add action to open "Configure Annotation" settings dialog
-    KActionCollection *ac = qobject_cast<PageView *>(q->parent()->parent())->actionCollection();
-    QAction *aConfigAnnotation = ac->action(QStringLiteral("options_configure_annotations"));
-    if (aConfigAnnotation) {
-        aQuickTools->addAction(aConfigAnnotation);
+
+    // set the default action
+    if (quickTools.isEmpty()) {
+        aQuickTools->setDefaultAction(aQuickTools);
+        Okular::Settings::setQuickAnnotationDefaultAction(0);
+        Okular::Settings::self()->save();
+    } else {
+        int defaultAction = Okular::Settings::quickAnnotationDefaultAction();
+        if (isFirstTimePopulated && defaultAction < quickTools.count()) {
+            // we can reach here also if no quick tools were defined before, in that case defaultAction is correctly equal to zero
+            aQuickTools->setDefaultAction(quickTools.at(defaultAction));
+        } else {
+            // if the quick tools have been modified we cannot restore the previous default action
+            aQuickTools->setDefaultAction(quickTools.at(0));
+            Okular::Settings::setQuickAnnotationDefaultAction(0);
+            Okular::Settings::self()->save();
+        }
     }
 }
 
@@ -425,52 +458,28 @@ const QIcon AnnotationActionHandlerPrivate::stampIcon(const QString &stampIconNa
 
 void AnnotationActionHandlerPrivate::selectTool(int toolId)
 {
-    selectedTool = toolId;
-    annotator->selectTool(toolId, PageViewAnnotator::ShowTip::Yes);
+    selectedBuiltinTool = toolId;
+    annotator->selectBuiltinTool(toolId, PageViewAnnotator::ShowTip::Yes);
     parseTool(toolId);
 }
 
 void AnnotationActionHandlerPrivate::slotStampToolSelected(const QString &stamp)
 {
-    KMessageBox::information(nullptr, i18nc("@info", "Stamps inserted in PDF documents are not visible in PDF readers other than Okular"), i18nc("@title:window", "Experimental feature"), QStringLiteral("stampAnnotationWarning"));
-    selectedTool = PageViewAnnotator::STAMP_TOOL_ID;
+    ephemeralStampWarning();
+    selectedBuiltinTool = PageViewAnnotator::STAMP_TOOL_ID;
     annotator->selectStampTool(stamp); // triggers a reparsing thus calling parseTool
 }
 
 void AnnotationActionHandlerPrivate::slotQuickToolSelected(int favToolId)
 {
-    int toolId = annotator->setQuickTool(favToolId); // always triggers an unuseful reparsing
-    if (toolId == -1) {
-        qWarning("Corrupted configuration for quick annotation tool with id: %d", favToolId);
-        return;
+    if (isQuickToolStamp(favToolId)) {
+        ephemeralStampWarning();
     }
-    int indexOfActionInGroup = toolId - 1;
-    if (toolId == PageViewAnnotator::STAMP_TOOL_ID) {
-        // if the quick tool is a stamp we need to find its corresponding built-in tool action and select it
-        QDomElement favToolElement = annotator->quickTool(favToolId);
-        QDomElement engineElement = favToolElement.firstChildElement(QStringLiteral("engine"));
-        QDomElement annotationElement = engineElement.firstChildElement(QStringLiteral("annotation"));
-        QString stampIconName = annotationElement.attribute(QStringLiteral("icon"));
-
-        const auto defaultStamps = StampAnnotationWidget::defaultStamps();
-        auto it = std::find_if(defaultStamps.begin(), defaultStamps.end(), [&stampIconName](const QPair<QString, QString> &element) { return element.second == stampIconName; });
-        if (it != defaultStamps.end()) {
-            int stampActionIndex = std::distance(defaultStamps.begin(), it);
-            indexOfActionInGroup = PageViewAnnotator::STAMP_TOOL_ID + stampActionIndex - 1;
-        } else {
-            maybeUpdateCustomStampAction(stampIconName);
-            indexOfActionInGroup = agTools->actions().size() - 1;
-        }
-    }
-    QAction *favToolAction = agTools->actions().at(indexOfActionInGroup);
-    if (!favToolAction->isChecked()) {
-        // action group workaround: activates the action slot calling selectTool
-        //                          when new tool if different from the selected one
-        favToolAction->trigger();
-    } else {
-        selectTool(toolId);
-    }
-    aShowToolBar->setChecked(true);
+    annotator->selectQuickTool(favToolId);
+    selectedBuiltinTool = -1;
+    updateConfigActions();
+    Okular::Settings::setQuickAnnotationDefaultAction(favToolId - 1);
+    Okular::Settings::self()->save();
 }
 
 void AnnotationActionHandlerPrivate::slotSetColor(AnnotationColor colorType, const QColor &color)
@@ -503,11 +512,30 @@ void AnnotationActionHandlerPrivate::slotSelectAnnotationFont()
 
 void AnnotationActionHandlerPrivate::slotToolBarVisibilityChanged(bool checked)
 {
-    if (!checked) {
+    if (!checked && !isQuickToolAction(agTools->checkedAction())) {
         q->deselectAllAnnotationActions();
     }
 }
 
+bool AnnotationActionHandlerPrivate::isQuickToolAction(QAction *aTool)
+{
+    return aQuickTools->menu()->actions().contains(aTool) && aTool->isCheckable();
+}
+
+bool AnnotationActionHandlerPrivate::isQuickToolStamp(int toolId)
+{
+    QDomElement toolElement = annotator->quickTool(toolId);
+    const QString annotType = toolElement.attribute(QStringLiteral("type"));
+    QDomElement engineElement = toolElement.firstChildElement(QStringLiteral("engine"));
+    QDomElement annElement = engineElement.firstChildElement(QStringLiteral("annotation"));
+    return annotType == QStringLiteral("stamp");
+}
+
+void AnnotationActionHandlerPrivate::ephemeralStampWarning()
+{
+    KMessageBox::information(nullptr, i18nc("@info", "Stamps inserted in PDF documents are not visible in PDF readers other than Okular"), i18nc("@title:window", "Experimental feature"), QStringLiteral("stampAnnotationWarning"));
+}
+
 AnnotationActionHandler::AnnotationActionHandler(PageViewAnnotator *parent, KActionCollection *ac)
     : QObject(parent)
     , d(new AnnotationActionHandlerPrivate(this))
@@ -515,9 +543,12 @@ AnnotationActionHandler::AnnotationActionHandler(PageViewAnnotator *parent, KAct
     d->annotator = parent;
 
     // toolbar visibility actions
-    d->aShowToolBar = new KToggleAction(QIcon::fromTheme(QStringLiteral("draw-freehand")), i18n("&Annotations"), this);
+    d->aToolBarVisibility = new KToggleAction(QIcon::fromTheme(QStringLiteral("draw-freehand")), i18n("&Annotations"), this);
     d->aHideToolBar = new QAction(QIcon::fromTheme(QStringLiteral("dialog-close")), i18nc("@action:intoolbar Hide the toolbar", "Hide"), this);
-    connect(d->aHideToolBar, &QAction::triggered, this, [this]() { d->aShowToolBar->setChecked(false); });
+    connect(d->aHideToolBar, &QAction::triggered, this, [this]() { d->aToolBarVisibility->setChecked(false); });
+    d->aShowToolBar = new QAction(QIcon::fromTheme(QStringLiteral("draw-freehand")), i18nc("@action:intoolbar Show the builtin annotation toolbar", "Show more annotation tools"), this);
+    connect(d->aShowToolBar, &QAction::triggered, this, [this]() { d->aToolBarVisibility->setChecked(true); });
+    connect(d->aToolBarVisibility, &QAction::toggled, this, [this](bool checked) { d->aShowToolBar->setEnabled(!checked); });
 
     // Text markup actions
     KToggleAction *aHighlighter = new KToggleAction(QIcon::fromTheme(QStringLiteral("draw-highlight")), i18nc("@action:intoolbar Annotation tool", "Highlighter"), this);
@@ -535,15 +566,21 @@ AnnotationActionHandler::AnnotationActionHandler(PageViewAnnotator *parent, KAct
     KToggleAction *aRectangle = new KToggleAction(QIcon::fromTheme(QStringLiteral("draw-rectangle")), i18nc("@action:intoolbar Annotation tool", "Rectangle"), this);
     KToggleAction *aEllipse = new KToggleAction(QIcon::fromTheme(QStringLiteral("draw-ellipse")), i18nc("@action:intoolbar Annotation tool", "Ellipse"), this);
     KToggleAction *aPolygon = new KToggleAction(QIcon::fromTheme(QStringLiteral("draw-polyline")), i18nc("@action:intoolbar Annotation tool", "Polygon"), this);
-    d->aGeomShapes = new ToggleActionMenu(QIcon(), QString(), this, ToggleActionMenu::MenuButtonPopup, ToggleActionMenu::ImplicitDefaultAction);
-    d->aGeomShapes->setText(i18nc("@action", "Geometrical shapes"));
+    d->aGeomShapes = new ToggleActionMenu(i18nc("@action", "Geometrical shapes"), this);
     d->aGeomShapes->setEnabled(true); // Need to explicitly set this once, or refreshActions() in part.cpp will disable this action
+#if KWIDGETSADDONS_VERSION < QT_VERSION_CHECK(5, 77, 0)
+    d->aGeomShapes->setDelayed(false);
+    d->aGeomShapes->setStickyMenu(false);
+#else
+    d->aGeomShapes->setPopupMode(QToolButton::MenuButtonPopup);
+#endif
     d->aGeomShapes->addAction(aArrow);
     d->aGeomShapes->addAction(aStraightLine);
     d->aGeomShapes->addAction(aRectangle);
     d->aGeomShapes->addAction(aEllipse);
     d->aGeomShapes->addAction(aPolygon);
     d->aGeomShapes->setDefaultAction(aArrow);
+    connect(d->aGeomShapes->menu(), &QMenu::triggered, d->aGeomShapes, &ToggleActionMenu::setDefaultAction);
 
     // The order in which the actions are added is relevant to connect
     // them to the correct toolId defined in tools.xml
@@ -579,13 +616,15 @@ AnnotationActionHandler::AnnotationActionHandler(PageViewAnnotator *parent, KAct
     }
 
     // Stamp action
-    d->aStamp = new ToggleActionMenu(QIcon::fromTheme(QStringLiteral("tag")), QString(), this, ToggleActionMenu::MenuButtonPopup, ToggleActionMenu::ImplicitDefaultAction);
-    d->aStamp->setText(i18nc("@action", "Stamp"));
-
+    d->aStamp = new ToggleActionMenu(QIcon::fromTheme(QStringLiteral("tag")), i18nc("@action", "Stamp"), this);
+#if KWIDGETSADDONS_VERSION < QT_VERSION_CHECK(5, 77, 0)
+    d->aStamp->setDelayed(false);
+    d->aStamp->setStickyMenu(false);
+#else
+    d->aStamp->setPopupMode(QToolButton::MenuButtonPopup);
+#endif
     for (const auto &stamp : StampAnnotationWidget::defaultStamps()) {
         KToggleAction *ann = new KToggleAction(d->stampIcon(stamp.second), stamp.first, this);
-        if (!d->aStamp->defaultAction())
-            d->aStamp->setDefaultAction(ann);
         d->aStamp->addAction(ann);
         d->agTools->addAction(ann);
         // action group workaround: connecting to toggled instead of triggered
@@ -595,12 +634,36 @@ AnnotationActionHandler::AnnotationActionHandler(PageViewAnnotator *parent, KAct
                 d->slotStampToolSelected(stamp.second);
         });
     }
+    if (!d->aStamp->menu()->actions().isEmpty()) {
+        d->aStamp->setDefaultAction(d->aStamp->menu()->actions().first());
+    }
+    connect(d->aStamp->menu(), &QMenu::triggered, d->aStamp, &ToggleActionMenu::setDefaultAction);
 
     // Quick annotations action
-    d->aQuickTools = new KSelectAction(QIcon::fromTheme(QStringLiteral("draw-freehand")), i18nc("@action:intoolbar Show list of quick annotation tools", "Quick Annotations"), this);
+    d->aQuickTools = new ToggleActionMenu(i18nc("@action:intoolbar Show list of quick annotation tools", "Quick Annotations"), this);
+#if KWIDGETSADDONS_VERSION < QT_VERSION_CHECK(5, 77, 0)
+    d->aQuickTools->setDelayed(false);
+    d->aQuickTools->setStickyMenu(false);
+#else
+    d->aQuickTools->setPopupMode(QToolButton::MenuButtonPopup);
+#endif
+    d->aQuickTools->setIcon(QIcon::fromTheme(QStringLiteral("draw-freehand")));
     d->aQuickTools->setToolTip(i18nc("@info:tooltip", "Choose an annotation tool from the quick annotations"));
-    d->aQuickTools->setToolBarMode(KSelectAction::MenuMode);
     d->aQuickTools->setEnabled(true); // required to ensure that populateQuickAnnotations is executed the first time
+    // set the triggered quick annotation as default action (but avoid setting 'Configure...' as default action)
+    connect(d->aQuickTools->menu(), &QMenu::triggered, this, [this](QAction *action) {
+        if (action->isCheckable()) {
+            d->aQuickTools->setDefaultAction(action);
+        }
+    });
+    QAction *aQuickToolsSeparator = new QAction(this);
+    aQuickToolsSeparator->setSeparator(true);
+    d->aQuickTools->addAction(aQuickToolsSeparator);
+    d->aQuickTools->addAction(d->aShowToolBar);
+    QAction *aConfigAnnotation = ac->action(QStringLiteral("options_configure_annotations"));
+    if (aConfigAnnotation) {
+        d->aQuickTools->addAction(aConfigAnnotation);
+    }
     d->populateQuickAnnotations();
 
     // Add to quick annotation action
@@ -655,12 +718,14 @@ AnnotationActionHandler::AnnotationActionHandler(PageViewAnnotator *parent, KAct
             d->selectTool(-1);
         } else {
             d->agLastAction = action;
-            // Show the annotation toolbar whenever actions are triggered (e.g using shortcuts)
-            d->aShowToolBar->setChecked(true);
+            // Show the annotation toolbar whenever builtin tool actions are triggered (e.g using shortcuts)
+            if (!d->isQuickToolAction(action)) {
+                d->aToolBarVisibility->setChecked(true);
+            }
         }
     });
 
-    ac->addAction(QStringLiteral("mouse_toggle_annotate"), d->aShowToolBar);
+    ac->addAction(QStringLiteral("mouse_toggle_annotate"), d->aToolBarVisibility);
     ac->addAction(QStringLiteral("hide_annotation_toolbar"), d->aHideToolBar);
     ac->addAction(QStringLiteral("annotation_highlighter"), aHighlighter);
     ac->addAction(QStringLiteral("annotation_underline"), aUnderline);
@@ -688,18 +753,18 @@ AnnotationActionHandler::AnnotationActionHandler(PageViewAnnotator *parent, KAct
     ac->addAction(QStringLiteral("annotation_settings_font"), d->aFont);
     ac->addAction(QStringLiteral("annotation_settings_advanced"), d->aAdvancedSettings);
 
-    ac->setDefaultShortcut(d->aShowToolBar, Qt::Key_F6);
-    ac->setDefaultShortcut(aHighlighter, Qt::ALT | Qt::Key_1);
-    ac->setDefaultShortcut(aUnderline, Qt::ALT | Qt::Key_2);
-    ac->setDefaultShortcut(aSquiggle, Qt::ALT | Qt::Key_3);
-    ac->setDefaultShortcut(aStrikeout, Qt::ALT | Qt::Key_4);
-    ac->setDefaultShortcut(aTypewriter, Qt::ALT | Qt::Key_5);
-    ac->setDefaultShortcut(aInlineNote, Qt::ALT | Qt::Key_6);
-    ac->setDefaultShortcut(aPopupNote, Qt::ALT | Qt::Key_7);
-    ac->setDefaultShortcut(aFreehandLine, Qt::ALT | Qt::Key_8);
-    ac->setDefaultShortcut(aArrow, Qt::ALT | Qt::Key_9);
-    ac->setDefaultShortcut(aRectangle, Qt::ALT | Qt::Key_0);
-    ac->setDefaultShortcut(d->aAddToQuickTools, QKeySequence(Qt::CTRL | Qt::SHIFT | Qt::Key_B));
+    ac->setDefaultShortcut(d->aToolBarVisibility, Qt::Key_F6);
+    ac->setDefaultShortcut(aHighlighter, Qt::ALT + Qt::Key_1);
+    ac->setDefaultShortcut(aUnderline, Qt::ALT + Qt::Key_2);
+    ac->setDefaultShortcut(aSquiggle, Qt::ALT + Qt::Key_3);
+    ac->setDefaultShortcut(aStrikeout, Qt::ALT + Qt::Key_4);
+    ac->setDefaultShortcut(aTypewriter, Qt::ALT + Qt::Key_5);
+    ac->setDefaultShortcut(aInlineNote, Qt::ALT + Qt::Key_6);
+    ac->setDefaultShortcut(aPopupNote, Qt::ALT + Qt::Key_7);
+    ac->setDefaultShortcut(aFreehandLine, Qt::ALT + Qt::Key_8);
+    ac->setDefaultShortcut(aArrow, Qt::ALT + Qt::Key_9);
+    ac->setDefaultShortcut(aRectangle, Qt::ALT + Qt::Key_0);
+    ac->setDefaultShortcut(d->aAddToQuickTools, QKeySequence(Qt::CTRL + Qt::SHIFT + Qt::Key_B));
     d->updateConfigActions();
 }
 
@@ -712,7 +777,7 @@ AnnotationActionHandler::~AnnotationActionHandler()
 void AnnotationActionHandler::setupAnnotationToolBarVisibilityAction()
 {
     // find the main window associated to the toggle toolbar action
-    QList<QWidget *> widgets = d->aShowToolBar->associatedWidgets();
+    QList<QWidget *> widgets = d->aToolBarVisibility->associatedWidgets();
     auto itMainWindow = std::find_if(widgets.begin(), widgets.end(), [](const QWidget *widget) { return qobject_cast<const KParts::MainWindow *>(widget) != nullptr; });
     Q_ASSERT(itMainWindow != widgets.end());
     KParts::MainWindow *mw = qobject_cast<KParts::MainWindow *>(*itMainWindow);
@@ -721,15 +786,17 @@ void AnnotationActionHandler::setupAnnotationToolBarVisibilityAction()
     auto itToolBar = std::find_if(toolbars.begin(), toolbars.end(), [](const KToolBar *toolBar) { return toolBar->objectName() == QStringLiteral("annotationToolBar"); });
     Q_ASSERT(itToolBar != toolbars.end());
     KToolBar *annotationToolBar = mw->toolBar(QStringLiteral("annotationToolBar"));
-    d->aShowToolBar->setChecked(annotationToolBar->isVisible());
-    connect(annotationToolBar, &QToolBar::visibilityChanged, d->aShowToolBar, &QAction::setChecked, Qt::UniqueConnection);
-    connect(d->aShowToolBar, &QAction::toggled, annotationToolBar, &KToolBar::setVisible, Qt::UniqueConnection);
-    connect(d->aShowToolBar, &QAction::toggled, this, [this](bool checked) { d->slotToolBarVisibilityChanged(checked); });
+    d->aToolBarVisibility->setChecked(annotationToolBar->isVisible());
+    connect(annotationToolBar, &QToolBar::visibilityChanged, d->aToolBarVisibility, &QAction::setChecked, Qt::UniqueConnection);
+    connect(d->aToolBarVisibility, &QAction::toggled, annotationToolBar, &KToolBar::setVisible, Qt::UniqueConnection);
+    connect(d->aToolBarVisibility, &QAction::toggled, this, [this](bool checked) { d->slotToolBarVisibilityChanged(checked); });
+
+    d->aShowToolBar->setEnabled(!annotationToolBar->isVisible());
 }
 
 void AnnotationActionHandler::reparseBuiltinToolsConfig()
 {
-    d->parseTool(d->selectedTool);
+    d->parseTool(d->selectedBuiltinTool);
 }
 
 void AnnotationActionHandler::reparseQuickToolsConfig()
